unit MainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls;

type
  TMainDlg = class(TForm)
    ConvertBtn: TButton;
    ResultLabel: TLabel;
    Label1: TLabel;
    FamilyMemo: TMemo;
    Label2: TLabel;
    TypeMemo: TMemo;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    procedure ConvertBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
  public
  end;

var
  MainDlg: TMainDlg;

implementation

{$R *.dfm}

uses
  StdConvs, ConvUtils;

type
  TConvTypeBeerForJim = class(TConvTypeInfo)
  private
    FGlassSize   : Double;
    FPercAlcohol : Double;
  protected
    function GetAlcoholAmt : Double;
  public
    constructor Create(const AConvFamily : TConvFamily;
                       const ADescription : string;
                       const AGlassSize   : Double;
                       const APercAlcohol : Double);
    function ToCommon(const AValue : Double) : Double; override;
    function FromCommon(const AValue : Double) : Double; override;
  end;


var
  // Song Time family
  cbSongTime : TConvFamily;

  suRamones    : TConvType;
  suDeepPurple : TConvType;
  suELP: TConvType;

  // Beer for Jim family
  cbBeerForJim      : TConvFamily;
  buHoegaarden      : TConvType;
  buDeliriumTremens : TConvType;
  buOldPeculier     : TCOnvType;

  { TConvTypeBeerForJim }

constructor TConvTypeBeerForJim.Create(const AConvFamily : TConvFamily;
                                       const ADescription : string;
                                       const AGlassSize   : Double;
                                       const APercAlcohol : Double);
begin
  inherited Create(AConvFamily,ADescription);
  FGlassSize   := AGlassSize;
  FPercAlcohol := APercAlcohol;
end;


function TConvTypeBeerForJim.GetAlcoholAmt : Double;
begin
  // Calculate the amount of alcohol in a glass of beer
  Result := FGlassSize * FPercAlcohol / 100;
end;


function TConvTypeBeerForJim.FromCommon(const AValue : Double) : Double;
begin
  // Given AValue millilitres of alcohol, calculate the number of glasses
  // of the current beer that are needed to contain that much alcohol
  Result := AValue / GetAlcoholAmt;
end;


function TConvTypeBeerForJim.ToCommon(const AValue : Double) : Double;
begin
  // Given AValue glasses of beer, get the number of milllitres of alcohol
  Result := GetAlcoholAmt * AValue;
end;


function RegisterBeerConversionType(const Description : string;
                                    const GlassSize,PercAlcohol : Double) : TConvType;
  var
    Temp : TConvTypeInfo;
begin
  Temp := TConvTypeBeerForJim.Create(cbBeerForJim,Description,GlassSize,PercAlcohol);

  if not RegisterConversionType(Temp,Result) then begin
    Temp.Free;
    RaiseConversionRegError(cbBeerForJim,Description);
  end;
end;


{ TMainDlg }

procedure GetFamilies(AList : TStrings);
  var
    FamilyArray : TConvFamilyArray;
    i           : Integer;
begin
  AList.CLear;
  GetConvFamilies(FamilyArray);

  for i := Low(FamilyArray) to High(FamilyArray) do begin
    AList.Add(ConvFamilyToDescription(FamilyArray[i]))
  end;
end;


procedure GetTypes(AFamily : TConvFamily;AList : TStrings);
  var
    TypeArray : TConvTypeArray;
    i         : Integer;
begin
  AList.CLear;
  GetConvTypes(AFamily,TypeArray);

  for i := Low(TypeArray) to High(TypeArray) do begin
    AList.Add(ConvTypeToDescription(TypeArray[i]))
  end;
end;


function ParseBeerString(const InString : string) : string;
  var
    GlassesOfBeer : Double;
    TempType      : TConvType;
begin
  if TryStrToConvUnit(InString,GlassesOfBeer,TempType) then begin
    Result := FormatFloat('0.000',GlassesOfBeer) + ' glasses of ' +
              ConvTypeToDescription(TempType);
  end else begin
    Result := 'Error in beer conversion. Go directly to bar!';
  end
end;


procedure TMainDlg.FormCreate(Sender: TObject);
begin
  GetFamilies(FamilyMemo.Lines);
  GetTypes(cbBeerForJim,TypeMemo.Lines);
  Label3.Caption := ParseBeerString('2');
  Label4.Caption := ParseBeerString('2 hoegaarden');
  Label5.Caption := ParseBeerString('2 frogs');
end;


procedure TMainDlg.ConvertBtnClick(Sender: TObject);
begin
//  ResultLabel.Caption := FormatFloat('0.000',Convert(1,duCubits,tuCelsius));
//  ResultLabel.Caption := FormatFloat('0.000',Convert(10,duMiles,tuHours,duMeters,tuSeconds));
//  ResultLabel.Caption := FormatFloat('0.000',Convert(1,suRamones,suELP));
  ResultLabel.Caption := FormatFloat('0.000',Convert(1,buHoegaarden,buOldPeculier));
end;


initialization
  cbSongTime   := RegisterConversionFamily('Song Times');
  suRamones    := RegisterConversionType(cbSongTime,'Ramones songs',1);
  suDeepPurple := RegisterConversionType(cbSongTime,'Deep Purple songs',1.705);
  suELP        := RegisterConversionType(cbSongTime,'ELP songs',6.098);

  cbBeerForJim      := RegisterConversionFamily('Beers for Jim');
  buHoegaarden      := RegisterBeerConversionType('Hoegaarden',330,5);
  buDeliriumTremens := RegisterBeerConversionType('Delirium Tremens',330,9);
  buOldPeculier     := RegisterBeerConversionType('Old Peculier',330,5.7);

end.
